@@ -1,11 +1,11 @@
 ##############################################################################
 # Net::Twitter - Perl OO interface to www.twitter.com
-# v2.01
+# v2.02
 # Copyright (c) 2009 Chris Thompson
 ##############################################################################
 
 package Net::Twitter;
-$VERSION = "2.01";
+$VERSION = "2.02";
 use strict;
 
 use URI::Escape;
@@ -98,6 +98,9 @@ sub new {
     $conf{skip_arg_validation} = 0 unless defined $conf{skip_arg_validation};
     $conf{die_on_validation}   = 0 unless defined $conf{die_on_validation};
 
+    $conf{error_return_val} =
+      ( ( defined $conf{arrayref_on_error} ) and ( $conf{arrayref_on_error} ) ) ? [] : undef;
+
     $conf{response_error}  = undef;
     $conf{response_code}   = undef;
     $conf{response_method} = undef;
@@ -109,7 +112,10 @@ sub new {
 ### Parallel/Async setups like POE. Set response_error to undef to prevent
 ### spillover, just in case.
 
-sub clone { bless { %{ $_[0] }, response_error => undef } }
+sub clone {
+    my $self = shift;
+    bless { %{$self}, response_error => $self->{error_return_val} };
+}
 
 ### Change the credentials
 
@@ -140,23 +146,23 @@ sub http_message {
 sub update_twittervision {
     my ( $self, $location ) = @_;
     my $response;
- 
+
     if ( $self->{twittervision} ) {
-        my $tvreq = $self->{tvua}->post(
-            $self->{tvurl} . "/user/update_location.json",
-            [ location => uri_escape($location) ]
-        );
+        my $tvreq =
+          $self->{tvua}
+          ->post( $self->{tvurl} . "/user/update_location.json", [ location => uri_escape($location) ] );
         if ( $tvreq->content ne "User not found" ) {
             $self->{response_code}    = $tvreq->code;
             $self->{response_message} = $tvreq->message;
             $self->{response_error}   = $tvreq->content;
-            
+
             if ( $tvreq->is_success ) {
                 $response = eval { JSON::Any->jsonToObj( $tvreq->content ) };
 
                 if ( !defined $response ) {
                     $self->{response_error} =
-                      "TWITTERVISION RETURNED SUCCESS BUT PARSING OF THE RESPONSE FAILED - " . $tvreq->content;
+                      "TWITTERVISION RETURNED SUCCESS BUT PARSING OF THE RESPONSE FAILED - "
+                      . $tvreq->content;
                 }
             }
         }
@@ -173,7 +179,7 @@ sub search {
     ### Return if no args specified.
     if ( !defined($args) ) {
         carp "No query string specified";
-        return undef;
+        return $self->{error_return_val};
     }
 
     ### If the first argument passed is a string, convert it into a hashref
@@ -238,6 +244,7 @@ sub search {
         if ( !defined $retval ) {
             $self->{response_error} =
               "TWITTER RETURNED SUCCESS BUT PARSING OF THE RESPONSE FAILED - " . $req->content;
+            return $self->{error_return_val};
         }
     }
     return $retval;
@@ -539,7 +546,7 @@ BEGIN {
                         "error"   => "The method $whoami requires arguments and none specified."
                     };
                 }
-                return undef;
+                return $self->{error_return_val};
             }
 
             ### For backwards compatibility we need to handle the user handing a single, scalar
@@ -547,20 +554,24 @@ BEGIN {
             ### different defaults, use a bit of logic to stick the value in the right place.
 
             if ( ( !ref($args) ) && ( defined $args ) ) {
-                my $single_arg;
-                if ( $whoami eq "update" ) {
-                    $single_arg = "status";
+                if ( $whoami eq "relationship_exists" ) {
+                    my $user_b = shift;
+                    $args = { "user_a" => $args, "user_b" => $user_b };
+                } elsif ( $whoami eq "update" ) {
+                    $args = { "status" => $args };
                 } elsif ( $whoami eq "replies" ) {
-                    $single_arg = "page";
-                } elsif ( $whoami =~ m/create_block|destroy_block|friends\b|show_user|create_friend|destroy_friend|destroy_direct_message/) {
-                    $single_arg = "id";
+                    $args = { "page" => $args };
+                } elsif ( $whoami =~
+m/create_block|destroy_block|friends\b|show_user|create_friend|destroy_friend|destroy_direct_message/
+                  )
+                {
+                    $args = { "id" => $args };
                 } elsif ( $whoami =~ m/update_profile_image|update_profile_background_image/ ) {
-                    $single_arg = "image";
+                    $args = { "image" => $args };
                 } elsif ( $whoami eq "update_delivery_device" ) {
-                    $single_arg = "device";
+                    $args = { "device" => $args };
                 } elsif ( $whoami eq "update_location" ) {
-                    $single_arg = "location";
-
+                    $args = { "location" => $args };
                 } else {
                     ### $args is not a hashref and $whoami is not one of the legacy
                     ### subs, so we punt.
@@ -573,9 +584,8 @@ BEGIN {
                             "error"   => "Argument is not a HASHREF."
                         };
                     }
-                    return undef;
+                    return $self->{error_return_val};
                 }
-                $args = { $single_arg => $args };
             }
 
             ### Handle source arg for update method.
@@ -600,7 +610,7 @@ BEGIN {
                             "request" => $method_def->{uri},
                             "error"   => "Either id or email is required by show_user, discarding request.",
                         };
-                        return undef;
+                        return $self->{error_return_val};
                     }
                 } else {
 
@@ -613,7 +623,7 @@ BEGIN {
                             "request" => $method_def->{uri},
                             "error"   => "The field id is required and not specified",
                         };
-                        return undef;
+                        return $self->{error_return_val};
                     } else {
                         $url .= $method_def->{uri} . ".json";
                     }
@@ -649,7 +659,7 @@ BEGIN {
                                     "error"   => "The field $argname is required and not specified"
                                 };
                             }
-                            return undef;
+                            return $self->{error_return_val};
                         }
 
                     }
@@ -658,6 +668,29 @@ BEGIN {
 
             }
 
+            ### Final Validation of $args, Drop args that are named but undefined, and complain when
+            ### bogus args are passed.
+
+            if ( !$self->{skip_arg_validation} ) {
+                foreach my $argkey ( sort keys %{$args} ) {
+                    if ( !defined $args->{$argkey} ) {
+                        carp "Argument $argkey specified as undef, discarding.";
+                        $self->{response_error} = {
+                            "request" => $url,
+                            "error"   => "Argument $argkey specified as undef."
+                        };
+                        delete $args->{$argkey};
+                    } elsif ( ! exists $method_def->{args}->{$argkey} ) {
+                        carp "Unknown argument $argkey passed, discarding.";
+                        $self->{response_error} = {
+                            "request" => $url,
+                            "error"   => "Unknown argument $argkey passed."
+                        };
+	                    delete $args->{$argkey};
+                    }
+                }
+            }
+
             ### Send the LWP request
             my $req;
             if ( $reqtype eq 'POST' ) {
@@ -685,6 +718,7 @@ BEGIN {
                 if ( !defined $retval ) {
                     $self->{response_error} =
                       "TWITTER RETURNED SUCCESS BUT PARSING OF THE RESPONSE FAILED - " . $req->content;
+                    return $self->{error_return_val};
                 }
             }
 
@@ -703,7 +737,7 @@ Net::Twitter - Perl interface to twitter.com
 
 =head1 VERSION
 
-This document describes Net::Twitter version 2.01
+This document describes Net::Twitter version 2.02
 
 =head1 SYNOPSIS
 
@@ -857,6 +891,14 @@ listed below, and to return undef to the caller. Passing a boolean true value fo
 new() will change this behavior to simply executing a die() with the appropriate error message. This
 defaults to false.
 
+=item C<arrayref_on_error>
+
+OPTIONAL: By default any methods which find an error, whether from twitter or from bad args, will 
+return undef. Passing C<arrayref_on_error> as a boolean TRUE to new() will cause all error states to
+return an empty arrayref instead. As most successful responses are in the form of arrayrefs, this will
+cause a uniform response type for all calls. All error messages and codes are still available with
+methods such as C<get_error>.
+
 =back
 
 =item C<clone()>
